package cn.bunny.service.web.security;

import cn.bunny.dto.LoginDto;
import cn.bunny.entity.system.User;
import cn.bunny.entity.system.UserRole;
import cn.bunny.security.custom.CustomUser;
import cn.bunny.service.web.mapper.UserMapper;
import cn.bunny.service.web.mapper.UserRoleMapper;
import cn.bunny.service.web.service.LoginService;
import cn.bunny.vo.system.LoginVo;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;

import java.util.List;

@Configuration
public class CustomUserDetailsService implements cn.bunny.security.service.CustomUserDetailsService {
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private LoginService loginService;
    @Autowired
    private UserRoleMapper userRoleMapper;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        // 根据邮箱查询用户名
        User user = userMapper.selectOne(Wrappers.<User>lambdaQuery().eq(User::getEmail, username));
        List<UserRole> sysRoleList = userRoleMapper.selectList(null);
        // 如果这次为空，再试下查询昵称
        if (user == null) {
            user = userMapper.selectOne(Wrappers.<User>lambdaQuery().eq(User::getNickName, username));
        }
        // 都为空抛出异常，用户不存在
        if (user == null) {
            throw new UsernameNotFoundException("");
        }

        // 查询所有的角色
        List<String> roleAuthoritieList = sysRoleList.stream().map(UserRole::getRoleCode).toList();
        return new CustomUser(user, AuthorityUtils.createAuthorityList(roleAuthoritieList));
    }

    /**
     * 前台用户登录接口
     *
     * @param loginDto 登录参数
     * @return 登录后结果返回
     */
    @Override
    public LoginVo login(LoginDto loginDto) {
        return loginService.login(loginDto);
    }
}
