package cn.bunny.service.web.service.impl;

import cn.bunny.common.result.constant.CommonMessageConstant;
import cn.bunny.common.service.utils.EmptyUtil;
import cn.bunny.common.service.utils.JwtHelper;
import cn.bunny.dto.LoginDto;
import cn.bunny.entity.system.User;
import cn.bunny.service.web.mapper.UserMapper;
import cn.bunny.service.web.service.LoginService;
import cn.bunny.vo.system.LoginVo;
import cn.bunny.vo.system.ValidateCodeVo;
import cn.hutool.captcha.CaptchaUtil;
import cn.hutool.captcha.CircleCaptcha;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class LoginServiceImpl extends ServiceImpl<UserMapper, User> implements LoginService {
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    /**
     * 前台用户登录接口
     *
     * @param loginDto 登录参数
     * @return 登录后结果返回
     */
    @Override
    public LoginVo login(LoginDto loginDto) {
        // 判断用户和密码是否为空
        String username = loginDto.getUsername();
        EmptyUtil.isEmpty(username, CommonMessageConstant.USERNAME_NOT_EMPTY);
        // 密码不能为空
        EmptyUtil.isEmpty(loginDto.getPassword(), CommonMessageConstant.PASSWORD_NOT_EMPTY);
        String password = DigestUtils.md5DigestAsHex(loginDto.getPassword().getBytes());

        // 设置返回类型
        LoginVo loginVo = userMapper.login(username, password);
        String token = JwtHelper.createToken(Long.valueOf(loginVo.getUserId()), loginVo.getEmail(), 7);
        loginVo.setToken(token);

        return loginVo;
    }

    /**
     * 生成验证码
     *
     * @return 验证码生成信息
     */
    @Override
    public ValidateCodeVo checkCode() {
        // 生成验证码
        CircleCaptcha captcha = CaptchaUtil.createCircleCaptcha(150, 48, 4, 2);
        String code = captcha.getCode();
        String base64Image = "data:image/png;base64," + captcha.getImageBase64();
        // 存储到Redis中
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        redisTemplate.opsForValue().set(uuid, code, 10, TimeUnit.MINUTES);
        return ValidateCodeVo.builder().codeKey(uuid).codeValue(base64Image).build();
    }
}
